// Data/GfxPointsData.H - This file is part of NotedELN

/* NotedELN is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   NotedELN is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with NotedELN.  If not, see <http://www.gnu.org/licenses/>.
*/

// GfxPointsData.H

#ifndef GFXPOINTSDATA_H

#define GFXPOINTSDATA_H

#include "GfxData.h"
#include <QList>

class GfxPointsData: public GfxData {
  Q_OBJECT;
  /* Lists xx() and yy() are relative to pos(). */
public:
  GfxPointsData(Data *parent);
  virtual ~GfxPointsData();
  QList<double> const &xx() const;
  QList<double> const &yy() const;
  QPointF point(int) const;
  int size() const;
  void setPoint(int, QPointF, bool hush=false);
  void addPoint(QPointF, bool hush=false);
  void removePoint(int i);
  void clear();
protected:
  void loadMore(QVariantMap const &src);
  void saveMore(QVariantMap &dst) const;
private:
  QList<double> xx_;
  QList<double> yy_;
};

#endif
